import sys
import os

file_path = os.path.abspath(__file__)
end = file_path.index('mns') + 16
project_path = file_path[0:end]
sys.path.append(project_path)

from loguru import logger
from datetime import datetime
import mns_common.constant.extra_income_db_name as extra_income_db_name
import mns_common.component.cookie.cookie_info_service as cookie_info_service
import mns_common.api.em.real_time.east_money_stock_us_api as east_money_stock_us_api
import mns_common.utils.data_frame_util as data_frame_util
import mns_common.component.us.us_stock_etf_info_api as us_stock_etf_info_api
import mns_common.component.us.us_common_service as us_common_service
mongodb_util = us_common_service.get_us_mongo_db()


# 同步东方财富美股信息
def sync_em_us_stock_etf_info(exist_em_df):
    # 同步东方财富美股信息 todo 增加稳定接口
    now_date = datetime.now()
    str_now_date = now_date.strftime('%Y-%m-%d %H:%M:%S')
    us_day = us_common_service.get_us_last_trade_day()
    logger.info("同步东方财富美股信息")
    em_cookie = cookie_info_service.get_em_cookie()
    if data_frame_util.is_not_empty(exist_em_df):
        us_stock_etf_info = exist_em_df.copy()
    else:
        us_stock_etf_info = east_money_stock_us_api.get_all_us_real_time_quotes_local_ip(30, em_cookie)

    us_stock_etf_info.loc[us_stock_etf_info['industry'] == '-', 'industry'] = '其他'
    us_stock_etf_info.loc[us_stock_etf_info['industry'] == '-', 'industry_code'] = 'US0'

    us_stock_etf_info['sync_time'] = str_now_date
    us_stock_etf_info['us_day'] = us_day
    us_stock_etf_info = us_stock_etf_info.fillna(0)
    # 美股代码使用中划线  东财使用的下划线

    us_stock_etf_info['market_code_str'] = us_stock_etf_info['market_code'].astype(str)
    us_stock_etf_info['em_symbol'] = us_stock_etf_info['market_code_str'] + '.' + us_stock_etf_info['symbol']
    us_stock_etf_info['symbol'] = us_stock_etf_info['symbol'].str.replace('_', '-')
    us_stock_etf_info['_id'] = us_stock_etf_info['symbol']

    us_stock_etf_info = us_stock_etf_info[[
        "_id",
        "symbol",
        "name",
        "em_symbol",
        "industry",
        "industry_code",
        "list_date",
        "sync_time",
        "us_day",
        "market_code",
        "now_price",
        "chg",
        "change_price",
        "volume",
        "amount",
        "exchange",
        "pe_ttm",
        "quantity_ratio",
        "high",
        "low",
        "open",
        "yesterday_price",
        "voucher_type",
        "total_mv",
        "flow_mv",
        "wei_bi",
        "outer_disk",
        "inner_disk",
        "today_main_net_inflow",
        "super_large_order_net_inflow",
        "super_large_order_net_inflow_ratio",
        "large_order_net_inflow",
        "concept_name_str",
        "today_main_net_inflow_ratio",
        "average_price",
        "concept_code_str",
        "large_order_net_inflow_ratio",
        "disk_ratio",
        "disk_diff_amount"
    ]]

    # voucher_type= 1 美国本地公司
    # voucher_type= 2 优先股
    # voucher_type= 3 在多个市场上市的股票 如 阿里巴巴 台积电
    # voucher_type= 4 信托or基金
    # voucher_type= 5 ETF
    # voucher_type= 6  票据
    # voucher_type= 8   权证
    # voucher_type= 9 债务证券
    # voucher_type= 10 持股拥有完整投票权 BRK.A (伯克希尔哈撒韦 - A)：公司原始股票
    # BRK.B (伯克希尔哈撒韦 - B)：1996 年为满足中小投资者需求而发行，后经 2010 年 1 拆 50，进一步降低投资门槛，被称为 “Baby Berkshire”。
    us_stock_df = us_stock_etf_info.loc[us_stock_etf_info['voucher_type'].isin([1, 2, 3, 10])]
    us_etf_df = us_stock_etf_info.loc[~us_stock_etf_info['voucher_type'].isin([1, 2, 3, 10])]
    if data_frame_util.is_not_empty(us_stock_df):
        us_stock_df.reset_index(drop=True, inplace=True)
        mongodb_util.save_mongo(us_stock_df, extra_income_db_name.US_STOCK_INFO_EM)
        us_stock_df['_id'] = us_stock_df['symbol']+'_'+us_day
        mongodb_util.save_mongo(us_stock_df, extra_income_db_name.US_STOCK_INFO_EM+'_his')

    if data_frame_util.is_not_empty(us_etf_df):
        us_etf_df.reset_index(drop=True, inplace=True)
        del us_etf_df['industry']
        del us_etf_df['industry_code']
        del us_etf_df['concept_code_str']
        del us_etf_df['concept_name_str']
        del us_etf_df['pe_ttm']
        mongodb_util.save_mongo(us_etf_df, extra_income_db_name.US_ETF_INFO_EM)

        us_etf_df['_id'] = us_etf_df['symbol'] + '_' + us_day
        mongodb_util.save_mongo(us_etf_df, extra_income_db_name.US_ETF_INFO_EM + '_his')


# 同步alpha_vantage美股信息
def sync_us_alpha_stock_etf_list():
    now_date = datetime.now()
    str_now_date = now_date.strftime('%Y-%m-%d %H:%M:%S')
    us_day = us_common_service.get_us_last_trade_day()
    logger.info("同步alpha_vantage美股信息")

    us_alpha_stock_list_df = us_stock_etf_info_api.get_us_alpha_stock_list('SRMP19KUP24M8B0E')
    us_alpha_stock_list_df['_id'] = us_alpha_stock_list_df['symbol']

    us_alpha_stock_list_df['sync_time'] = str_now_date
    us_alpha_stock_list_df['us_day'] = us_day
    us_alpha_stock_list_df = us_alpha_stock_list_df.fillna(0)
    alpha_us_etf_info = us_alpha_stock_list_df.loc[us_alpha_stock_list_df['assetType'] == 'ETF']
    alpha_us_stock_info = us_alpha_stock_list_df.loc[us_alpha_stock_list_df['assetType'] == 'Stock']

    if data_frame_util.is_not_empty(alpha_us_stock_info):
        alpha_us_stock_info.reset_index(drop=True, inplace=True)
        mongodb_util.save_mongo(alpha_us_stock_info, extra_income_db_name.US_STOCK_INFO_ALPHA_VANTAGE)

    if data_frame_util.is_not_empty(alpha_us_etf_info):
        alpha_us_etf_info.reset_index(drop=True, inplace=True)
        mongodb_util.save_mongo(alpha_us_etf_info, extra_income_db_name.US_ETF_INFO_ALPHA_VANTAGE)


# 同步alpha_vantage美股退市信息
def sync_us_alpha_de_list():
    now_date = datetime.now()
    str_now_date = now_date.strftime('%Y-%m-%d %H:%M:%S')
    us_day = us_common_service.get_us_last_trade_day()
    logger.info("同步alpha_vantage美股退市信息")
    us_alpha_stock_de_list = us_stock_etf_info_api.get_us_alpha_stock_de_list('SRMP19KUP24M8B0E')
    us_alpha_stock_de_list['_id'] = us_alpha_stock_de_list['symbol']
    us_alpha_stock_de_list['sync_time'] = str_now_date
    us_alpha_stock_de_list['us_day'] = us_day
    us_alpha_stock_de_list = us_alpha_stock_de_list.fillna(0)

    if data_frame_util.is_not_empty(us_alpha_stock_de_list):
        us_alpha_stock_de_list.reset_index(drop=True, inplace=True)
        mongodb_util.save_mongo(us_alpha_stock_de_list, extra_income_db_name.US_DE_LIST_INFO_ALPHA_VANTAGE)


if __name__ == '__main__':
    sync_em_us_stock_etf_info(None)
    sync_us_alpha_stock_etf_list()
    sync_us_alpha_de_list()



import sys
import os

file_path = os.path.abspath(__file__)
end = file_path.index('mns') + 16
project_path = file_path[0:end]
sys.path.append(project_path)
from mns_common.db.v2.MongodbUtilV2 import MongodbUtilV2
import mns_common.constant.extra_income_db_name as extra_income_db_name
from datetime import datetime, timedelta
mongodbUtilV2_27019 = MongodbUtilV2('27019', extra_income_db_name.US_STOCK)
from mns_common.db.MongodbUtil import MongodbUtil
mongodb_util = MongodbUtil('27017')
import mns_common.utils.ip_util as ip_util

# 保存数据
def save_us_minute_data(us_df, col_name, symbol, insert_flag):
    us_df['symbol'] = symbol
    us_df['_id'] = symbol + '_' + us_df['time']
    us_df['close'] = round(us_df['close'], 3)
    us_df['high'] = round(us_df['high'], 3)
    us_df['low'] = round(us_df['low'], 3)
    us_df['open'] = round(us_df['open'], 3)
    us_df['volume'] = round(us_df['volume'], 0)
    us_df = us_df[[
        '_id',
        'open',
        'high',
        'low',
        'close',
        'volume',
        'symbol',
    ]]
    if insert_flag:
        mongodbUtilV2_27019.insert_mongo(us_df, col_name)
    else:
        mongodbUtilV2_27019.save_mongo(us_df, col_name)


# 0:星期一, 1:星期二, ..., 6:星期日
def get_us_last_trade_day():
    now_date = datetime.now()
    yesterday_date = now_date - timedelta(days=1)
    return yesterday_date.strftime('%Y-%m-%d')


def get_chinese_last_trade_day():
    now_date = datetime.now()
    return now_date.strftime('%Y-%m-%d')


# 是否是us stock day 不排除节假日
def is_us_trade_day():
    now_date = datetime.now()
    weekday_num = now_date.weekday()
    if 0 < weekday_num < 5:
        return True
    else:
        return False


# 获取us 数据库
def get_us_mongo_db():
    mac_address = ip_util.get_mac_address()
    if  mac_address is not None and mac_address == ip_util.APPLE_AIR_MAC_ADDRESS:
        return mongodb_util
    else:
        return mongodbUtilV2_27019


if __name__ == '__main__':
    trade_day = get_us_last_trade_day()
    print(trade_day)



import sys
import os

file_path = os.path.abspath(__file__)
end = file_path.index('mns') + 16
project_path = file_path[0:end]
sys.path.append(project_path)
import requests
from bs4 import BeautifulSoup

key_mapping = {
    '公司名称': 'company_mame',
    '公司简称': 'company_short_name',
    '英文名称': 'english_name',
    '所属行业': 'industry',
    '上市场所': 'listing_exchange',
    '公司网址': 'official_website',
    '公司总裁': 'ceo',
    '注册地址': 'registered_address',
    '员工人数': 'employee_count',
    '邮政编码': 'zip_code',
    '联系电话': 'phone_number',
    '联系传真': 'fax_number',
    '办公地址': 'office_address',
    '公司简介': 'company_profile',
    '公司LOGO': 'company_logo_url',
    'LOGO备注': 'logo_remark'
}


# 同花顺 美股地址:https://basic.10jqka.com.cn/168/PINS/company.html

# 获取美股公司信息 https://stockpage.10jqka.com.cn/LMT/company/
def get_us_ths_company_info(stock_code):
    """
    根据股票代码，获取同花顺公司基本信息
    :param stock_code: 股票代码，例如 LMT / AAPL 字符串格式，核心传参参数
    :return: dict 结构化的公司基本信息，失败返回空字典
    """
    # 1. 基础配置：参数化拼接URL + 请求头（必须加，否则会被反爬拦截）
    base_url = f"https://stockpage.10jqka.com.cn/{stock_code}/company/"
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/120.0.0.0 Safari/537.36",
        "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8"
    }
    company_data = {}  # 存储最终的公司信息

    try:
        # 2. 发起GET请求获取网页源码
        response = requests.get(url=base_url, headers=headers, timeout=10)
        response.raise_for_status()  # 抛出请求异常（404/500等）
        response.encoding = "utf-8"  # 指定编码，防止中文乱码

        # 3. 解析HTML网页结构
        soup = BeautifulSoup(response.text, "html.parser")
        # 精准定位目标表格（唯一核心表格）
        table = soup.find("table", class_="companyinfo-tab")
        if not table:
            print(f"错误：未找到 {stock_code} 的公司信息表格")
            return company_data

        # 4. 提取所有表格内的文本内容+清洗数据
        all_tds = table.find_all("td")
        for td in all_tds:
            td_text = td.get_text(strip=True)  # 自动去除首尾空格/换行
            if not td_text:
                continue
            # 按【字段名：字段值】的格式拆分数据，封装到字典
            if "：" in td_text:
                key, value = td_text.split("：", 1)  # 只分割第一个冒号
                if key in key_mapping:
                    company_data[key_mapping[key]] = value

        # 单独提取：公司LOGO图片地址（补充信息，可选）
        logo_img = table.find("img")
        if logo_img:
            company_data[key_mapping['公司LOGO']] = logo_img.get("src", "")
            company_data[key_mapping['LOGO备注']] = logo_img.get("alt", "")

        # 特殊清洗：公司简介 去除多余空格/换行符，格式化展示
        if "公司简介" in company_data:
            company_data[key_mapping['公司简介']] = company_data["公司简介"].replace("\r", "").replace("\n", "").strip()



    except requests.exceptions.RequestException as e:
        print(f"请求失败：{e}")
    except Exception as e:
        print(f"解析数据异常：{e}")

    return company_data


def get_us_ths_company_info_v2(stock_code, market_code):
    """
    根据股票代码，获取同花顺公司基本信息
    :param stock_code: 股票代码，例如 LMT / AAPL 字符串格式，核心传参参数
    :return: dict 结构化的公司基本信息，失败返回空字典
    """
    # 1. 基础配置：参数化拼接URL + 请求头（必须加，否则会被反爬拦截）
    base_url = f"https://basic.10jqka.com.cn/{market_code}/{stock_code}/company.html"
    """
       根据股票代码爬取同花顺公司详情页的完整公司信息
       :param stock_code: 股票代码 如 PINS、LMT、NVDA
       :return: dict 清洗后的英文key结构化公司信息，异常返回空字典
       """
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/120.0.0.0 Safari/537.36",
        "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8"
    }

    result = {}

    try:
        res = requests.get(base_url, headers=headers, timeout=15)
        res.raise_for_status()  # 捕获404/500等请求错误

        # ========== 【核心修复1：解决编码乱码的关键代码】 ==========
        res.encoding = res.apparent_encoding  # 自动识别网页真实编码，替代手动指定utf-8

        soup = BeautifulSoup(res.text, 'html.parser')
        table = soup.find('table', class_='companyinfo-tab')
        if not table:
            print(f"错误：未找到{stock_code}的公司信息表格")
            return result

        # 提取LOGO
        logo_img = table.find('img')
        if logo_img:
            result[key_mapping['公司LOGO']] = clean_text(logo_img.get('src', ''))

        # ========== 【核心修复2：提取所有表格数据+超级清洗+兼容冒号】 ==========
        all_td = table.find_all('td')
        for td in all_td:
            # 提取td内所有文本，先做超级清洗
            td_text = clean_text(td.get_text())
            # 过滤空内容的td
            if not td_text:
                continue
            # 兼容：页面中同时存在【全角冒号 ：】和【半角冒号 :】，统一替换为全角再拆分
            td_text_unify = td_text.replace(":", "：")
            if '：' not in td_text_unify:
                continue
            # 拆分字段名和字段值，只拆分第一个冒号，防止值里有冒号导致拆分失败
            key, value = td_text_unify.split('：', 1)
            key = clean_text(key)
            value = clean_text(value)
            # 映射为英文key并存入
            if key in key_mapping:
                result[key_mapping[key]] = value

    except Exception as e:
        print(f"爬取/解析失败：{str(e)}")

    return result


def clean_text(content):
    """
    超级文本清洗函数 - 根治所有乱码/空白/占位符问题
    :param content: 待清洗的文本内容
    :return: 干净无乱码的纯文本
    """
    if not content:
        return ""
    # 1. 清理所有Unicode不可见空白符+占位符（覆盖所有网页乱码源）
    content = content.replace('\u00a0', '').replace('\xa0', '').replace('\u3000', '')
    content = content.replace('\r', ' ').replace('\n', ' ').replace('\t', '')
    # 2. 把多个连续空格 替换成 单个空格，避免内容中间的冗余空白
    while "  " in content:
        content = content.replace("  ", " ")
    # 3. 去除首尾空格，返回纯净文本
    return content.strip()


# ------------------- 核心调用：参数传参 测试 -------------------
if __name__ == "__main__":
    # 股票代码 作为参数传入函数，满足你的核心要求 ✅
    stock_code_param = "GEV"  # 这里可替换任意股票代码，例如：AAPL、MSFT
    result = get_us_ths_company_info(stock_code_param)

    result1 = get_us_ths_company_info_v2('META', '168')

    # 格式化打印结果
    print(f"===== {stock_code_param} 公司基本信息 =====")
    for k, v in result.items():
        print(f"{k}: {v}")
